#!/usr/bin/env bats
#
# Copyright © Cloud Linux GmbH & Cloud Linux Software, Inc 2010-2021 All Rights Reserved
#
# Licensed under CLOUD LINUX LICENSE AGREEMENT
# http://cloudlinux.com/docs/LICENCE.TXT
#
load helpers/test_helper;

PIDFILE=/var/run/lvestats.pid
start_lvestats="service lvestats start"
stop_lvestats="service lvestats stop"
restart_lvestats="service lvestats restart"

check_process_count() {
  local processes=$(ps aux)
  local result=$(echo "${processes}" | grep ${PIDFILE} | wc -l)
  echo ${result}
}

race_process_calc(){
  local expected_process=$1
  local found='False'
  for i in {1..5}; do
      local process_calc=$(check_process_count)
      if [ "$process_calc" -ne "$expected_process" ]; then
          echo "Try ${i}. Found ${process_calc} process; expected ${expected_process}.\n" >&2
          sleep 1
      else
          found='True'
          break
      fi
  done
  echo "${found}"
}

setup() {
  # stop server between tests
  run bash -c "${stop_lvestats}"
  rm -f /usr/share/lve-stats/plugins/TestPlugin.py
  # Wait 5 seconds while lve-stats daemon really stops
  race_process_calc 0 > /dev/null
}

teardown() {
  # restart server so stopping it in this test doesn't affect the following tests
  run bash -c "${restart_lvestats}"
  race_process_calc 3 > /dev/null

  # Remove TestPlugin.py on the last tear down call
  if [ "$BATS_TEST_NUMBER" -eq ${#BATS_TEST_NAMES[@]} ]; then
    rm -f /usr/share/lve-stats/plugins/TestPlugin.py
    service lvestats restart
    sleep 3
  fi
}

@test "check start single lvestats server" {
  run bash -c "${start_lvestats}"
  assert_success

  # check if process is really running
  PID=`cat ${PIDFILE}`
  assert_running ${PID}
  #expected 3 process
  local result=$(race_process_calc 3)
  assert_equal "True" "${result}"
}

@test "check start two lvestats servers" {
  run bash -c "${start_lvestats}"
  assert_success

  # check if process is really running
  OLD_PID=`cat ${PIDFILE}`
  assert_running ${OLD_PID}
  #expected 3 process
  local result=$(race_process_calc 3)
  assert_equal "True" "${result}"

  # second attempt to start, server must fail
  run bash -c "${start_lvestats}"

  # "service lvestats restart" command reports errors only on systemd OS (CL7, CL8)
  # so we check its retcode only only on these OS
  if [ ${CL_N} -lt 7 ]; then
      assert_failure
      assert_match "$OLD_PID. Exiting"
  fi

  # check if process is running after second attempt
  NEW_PID=`cat ${PIDFILE}`
  assert_running ${NEW_PID}
  assert_equal ${NEW_PID} ${OLD_PID}

  #expected 3 process
  local result=$(race_process_calc 3)
  assert_equal "True" "${result}"
}

@test "check restart lvestats server" {
  assert_equal 0 $(check_process_count)
  run bash -c "${start_lvestats}"
  assert_success

  # save first PID
  PID_OLD=`cat ${PIDFILE}`
  assert_running ${PID_OLD}
  #expected 3 process
  local result=$(race_process_calc 3)
  assert_equal "True" "${result}"

  # restart server
  run bash -c "${restart_lvestats}"
  assert_success

  # save new PID
  PID_NEW=`cat ${PIDFILE}`
  assert_running ${PID_NEW}
  #expected 3 process
  local result=$(race_process_calc 3)
  assert_equal "True" "${result}"

  # check if new pid is different
  assert_different ${PID_OLD} ${PID_NEW}
}

@test "check start & stop lvestats server" {
  run bash -c "${start_lvestats}"
  assert_success

  # check if process is really running
  PID=`cat ${PIDFILE}`
  assert_running ${PID}
  #expected 3 process
  local result=$(race_process_calc 3)
  assert_equal "True" "${result}"

  # check if process is not running after stopping server
  run bash -c "${stop_lvestats}"
  assert_success
  assert_not_running ${PID}
  assert_equal 0 $(check_process_count)
}

@test "check start & stop of blocked plugin" {
  cp files/lve_stats_server/TestPlugin.py /usr/share/lve-stats/plugins/TestPlugin.py
  run bash -c "lvestats-server start --pidfile ${PIDFILE} --plugins=TestPlugin"
  assert_success
  # check if process is really running
  PID=`cat ${PIDFILE}`
  assert_running ${PID}
  #expected 3 process
  local result=$(race_process_calc 3)
  assert_equal "True" "${result}"

  timestamp=`date +"%s"`
  # check if process is not running after stopping server
  run bash -c "lvestats-server stop --pidfile ${PIDFILE}"
  timestamp2=`date +"%s"`
  assert_success
  assert_not_running ${PID}
  assert_equal 0 $(check_process_count)
  assert_between $( expr ${timestamp2} - ${timestamp}) 9 16
}
